@@ -22,7 +22,7 @@ Key points:
 
 Usage:
 1. Ensure you have a LaTeX distribution (e.g., TeX Live) and xelatex installed.
-2. Create a preamble.tex and endtex.tex file in your configuration directory (~/.config/fintools/LaTeXBuilder/).
+2. Create a preamble.tex and endtex.tex file in your configuration directory (~/.config/fintoolsap/LaTeXBuilder/).
 3. Import this module into your Python script.
 4. Use the table_document or graph_document function to create LaTeX documents with tables or graphs.
 """
@@ -288,7 +288,7 @@ def _load_preamble(**kwargs) -> tuple[str, str]:
     """
 
     # create directory if not exist
-    user_config_dir = pathlib.Path(os.path.expanduser('~') + '/.config/fintools/LaTeXBuilder/')
+    user_config_dir = pathlib.Path(os.path.expanduser('~') + '/.config/fintoolsap/LaTeXBuilder/')
     os.makedirs(user_config_dir, exist_ok = True)
 
     # make sure required files are in directory
