@@ -12,6 +12,7 @@ import pandas
 import pathlib
 import datetime
 import functools
+import time
 import scipy.stats
 import numpy.typing
 import scipy.special
@@ -490,7 +491,7 @@ def group_quantile(df: pandas.DataFrame | polars.DataFrame,
                    vr: typing.Optional[typing.Union[str, list[str]]] = None, 
                    interpolation: typing.Optional[str] = 'linear',
                    no_merge: typing.Optional[bool] = False,
-                   set_index: typing.Optional[bool] = False
+                   set_index: typing.Optional[typing.Union[str, list[str]]] = None
                 ) -> pandas.DataFrame | polars.DataFrame:
     
     """
@@ -513,8 +514,8 @@ def group_quantile(df: pandas.DataFrame | polars.DataFrame,
         Interpolation method for quantile computation in polars (default is 'linear').
     no_merge : bool, optional
         If True, returns the quantile results without merging back to the original DataFrame.
-    set_index : bool, optional
-        If True, sets the group column(s) as the index in the returned DataFrame.
+    set_index : (str | list[str]), optional
+        If given, the index of the returned DataFrame will be set to these columns.
 
     Returns:
     --------
@@ -548,22 +549,24 @@ def group_quantile(df: pandas.DataFrame | polars.DataFrame,
             raise TypeError(f'vr: if qtiles is not a dictionary vr must be provided.')
     
     if not (isinstance(vr, dict) or isinstance(vr, list)):
-        raise TypeError(f'vr: expected type str or list[str], got {type(vars).__name__!r}')
+        raise TypeError(f'vr: expected type str or list[str], got {type(vr).__name__!r}')
     
     grouped: bool = gr is not None
     if grouped:
-        if not (isinstance(gr, dict) or isinstance(gr, list)):
-            raise TypeError(f'gr: expected type str or list[str], got {type(vars).__name__!r}')
-
+        if not (isinstance(gr, str) or isinstance(gr, list)):
+            raise TypeError(f'gr: expected type str or list[str], got {type(gr).__name__!r}')
+        if isinstance(gr, str):
+            gr = [gr]
+            
     dict_in: dict = {}
     if isinstance(qtiles, dict):
         dict_in = qtiles
     elif isinstance(qtiles, int):
         tiles: list = [i / qtiles for i in range(1, qtiles)]
-        for var in vars:
+        for var in vr:
             dict_in[var] = tiles
     else:
-        for var in vars:
+        for var in vr:
             dict_in[var] = qtiles
                 
     # compute quantiles
@@ -592,15 +595,21 @@ def group_quantile(df: pandas.DataFrame | polars.DataFrame,
                                    )
             fin = fin.reset_index(drop = False)
             if(no_merge):
-                if(set_index):
-                    fin = fin.set_index(gr)
-                    return(fin)
+                if(set_index is not None):
+                    if(isinstance(set_index, str)):
+                        set_index = [set_index]
+                    if(set_index):
+                        fin = fin.set_index(set_index)
+                        return(fin)
                 return(fin)
             else:
                 fin = df.merge(fin, how = 'inner', on = gr)
-                if(set_index):
-                    fin = fin.set_index(gr)
-                    return(fin)
+                if(set_index is not None):
+                    if(isinstance(set_index, str)):
+                        set_index = [set_index]
+                    if(set_index):
+                        fin = fin.set_index(set_index)
+                        return(fin)
                 return(fin)
         else:
             fin = functools.reduce(lambda x, y: pandas.merge(x, y, 
@@ -665,7 +674,7 @@ def group_nunique(df: polars.DataFrame | pandas.DataFrame,
                   name: typing.Optional[typing.Union[str, dict[str, str]]] = None, 
                   no_merge: typing.Optional[bool] = False, 
                   merge_how: typing.Optional[str] = 'left',
-                  set_index: typing.Optional[bool] = False
+                  set_index: typing.Optional[typing.Union[str, list[str]]] = None
                 ) -> polars.DataFrame | pandas.DataFrame:
         """
         Group by specified columns and calculate the number of unique values for each group.
@@ -685,8 +694,7 @@ def group_nunique(df: polars.DataFrame | pandas.DataFrame,
                     Pandas: {`left`, `inner`, `outer`, `cross`, `right`}
                     Polars: {`left`, `inner`, `outer`, `cross`, `asof`, `semi`, `anti`}
                 Defaults to 'left'.
-            set_index (bool, optional): Weather or not to set the index with `gr`. Default False.
-                Only applicable to pandas.DataFrame
+            set_index (str | list[str], optional): If given, the index of the returned DataFrame will be set to these columns.
 
         Returns:
             polars.DataFrame or pandas.DataFrame: DataFrame with the grouped results.
@@ -758,16 +766,22 @@ def group_nunique(df: polars.DataFrame | pandas.DataFrame,
                 res = res.rename(mapping = names)
 
         if(no_merge):
-            if(set_index and is_pandas):
-                res = res.set_index(gr)
-                return(res)
+            if(set_index is not None and is_pandas):
+                if(isinstance(set_index, str)):
+                    set_index = [set_index]
+                if(set_index):
+                    fin = res.set_index(set_index)
+                    return(fin)
             return(res)
         else:
             if(is_pandas):
                 res = df.merge(res, how = merge_how, on = gr)
-                if(set_index):
-                    res = res.set_index(gr)
-                    return(res)
+                if(set_index is not None):
+                    if(isinstance(set_index, str)):
+                        set_index = [set_index]
+                    if(set_index):
+                        fin = res.set_index(set_index)
+                        return(fin)
                 return(res)
             else:
                 res = df.join(res, how = merge_how, on = gr)
@@ -783,46 +797,101 @@ def group_avg(df: pandas.DataFrame | polars.DataFrame,
               merge_how: typing.Optional[str] = 'left',
               suppress_ouput: typing.Optional[bool] = False,
               desc: typing.Optional[str] = None,
-              set_index: typing.Optional[bool] = False
+              set_index: typing.Optional[typing.Union[str, list[str]]] = None
             ) -> pandas.DataFrame | polars.DataFrame:
         """
-        Compute group-wise averages in a pandas DataFrame.
-
-        Parameters:
-            - df (pandas.DataFrame or polars.DataFrame): The input DataFrame.
-            - gr (str or list[str]): The column(s) used for grouping.
-            - vr (str or list[str]): The column(s) whose average is to be
-                calculated.
-            - wt (str or None, optional): The weight column for weighted 
-                averages. Default is None. If None, equal weighted average 
-                is calculated.
-            - return_nan (bool, optional): If True, return numpy.nan in the
-                case of a ZeroDivideError else return equal weighted
-                mean. Default is False.
-            - name (str, dict[str: str] or None, optional): Name(s) to append 
-                to the resulting column(s). If str, it will be appended to 
-                all columns in 'vr'. If dict, keys are column names in 'vr' and 
-                values are the corresponding new names. Default is None.
-            - no_merge (bool, optional): If True, the resulting DataFrame 
-                will not be merged with the original DataFrame. Default is 
-                False. DataFrames are merged on gr.
-            - merge_how (str, optional): Type of merge operation when 
-                no_merge is False. Default is 'left'.
-            - suppress_output: (bool, optional): If True, the output from
-                tqdm.tqdm is suppressed when calculating the average of 
-                many columns for a pandas dataframe. Default is False.
-
-        Returns:
-            - pandas.DataFrame: A DataFrame with group-wise averages. 
-                If no_merge is False, it is the original DataFrame merged 
-                with the calculated averages.
-
-        Raises:
-            - TypeError: If the input arguments are not of the expected 
-                types.
-
-        Notes:
-            - Ignores numpy.nan in calculation of means.
+        Compute group-wise averages (optionally weighted) on a pandas or Polars DataFrame.
+    
+        Parameters
+        ----------
+        df : pandas.DataFrame or polars.DataFrame
+            Input DataFrame containing the data to aggregate.
+        gr : str or list of str
+            Column name or list of column names to group by.
+        vr : str or list of str
+            Column name or list of column names whose averages are computed.
+        wt : str, optional
+            Column name to use as weights for a weighted average. If None,
+            an unweighted (arithmetic) mean is used. Default is None.
+        ignore_nan : bool, optional
+            If True (default), then groups with zero total weight fall back to 
+            the unweighted mean; if False, such groups produce NaN.
+        name : str or dict, optional
+            If a string, appended to each output column name (e.g. `"_avg"`).
+            If a dict, maps original column names to desired output names.
+        no_merge : bool, optional
+            If True, return only the aggregated table; if False (default), merge
+            the aggregated columns back onto the original `df` on the group keys.
+        merge_how : str, optional
+            Merge strategy to use when `no_merge=False`. One of {'left', 'right', 
+            'inner', 'outer'}. Default is 'left'.
+        suppress_ouput : bool, optional
+            If True, disable progress bars during computation. Default is False.
+        desc : str, optional
+            Custom description to display in the progress bar prefix. Default is None.
+        set_index : str or list of str, optional
+            Column name(s) to set as the index of the returned pandas DataFrame.
+            Only applies when `no_merge=False` and `df` is a pandas DataFrame.
+            Default is None.
+    
+        Returns
+        -------
+        pandas.DataFrame or polars.DataFrame
+            If `no_merge=True`, returns a DataFrame with one row per group
+            containing the computed averages. Otherwise returns the original
+            `df` augmented with the new average columns.
+    
+        Raises
+        ------
+        TypeError
+            If `df` is not a pandas or Polars DataFrame, or if any of the other
+            parameters are of an invalid type (e.g. non-string column names,
+            incorrect `name` format, etc.).
+    
+        Examples
+        --------
+        >>> import pandas as pd
+        >>> df = pd.DataFrame({
+        ...     'group': ['A','A','B','B'],
+        ...     'value': [10, 20, 30, 40],
+        ...     'weight': [1, 2, 3, 4]
+        ... })
+        >>> # Unweighted group mean
+        >>> group_avg(df, gr='group', vr='value')
+          group  value
+        0     A   15.0
+        1     B   35.0
+    
+        >>> # Weighted group mean, with custom suffix
+        >>> group_avg(df, gr='group', vr='value', wt='weight', name='_wavg')
+          group  value_wavg
+        0     A   16.666667
+        1     B   36.666667
+    
+        >>> # Merge results back onto original DataFrame
+        >>> merged = group_avg(df, gr='group', vr='value', wt='weight', name='_wavg')
+        >>> merged.head()
+          group  value  weight  value_wavg
+        0     A     10       1    16.666667
+        1     A     20       2    16.666667
+        2     B     30       3    36.666667
+        3     B     40       4    36.666667
+    
+        >>> # Polars example
+        >>> import polars as pl
+        >>> pldf = pl.DataFrame(df)
+        >>> group_avg(pldf, gr='group', vr='value', wt='weight', name={'value':'avg_val'})
+        shape: (4, 3)
+        ┌───────┬───────┬─────────┐
+        │ group ┆ value ┆ avg_val │
+        │ ---   ┆ ---   ┆ ---     │
+        │ str   ┆ i64   ┆ f64     │
+        ╞═══════╪═══════╪═════════╡
+        │ A     ┆ 10    ┆ 16.6667 │
+        │ A     ┆ 20    ┆ 16.6667 │
+        │ B     ┆ 30    ┆ 36.6667 │
+        │ B     ┆ 40    ┆ 36.6667 │
+        └───────┴───────┴─────────┘
         """
         
         # type check df
@@ -908,7 +977,6 @@ def group_avg(df: pandas.DataFrame | polars.DataFrame,
             else:
                 dfs_to_merge = []
                 for col in tqdm.tqdm(vr, disable = suppress_ouput):
-                    pbar.set_description(f"{DESCRIPTION} {col}")
                     res = df.groupby(by = gr).apply(_wavg_py, col, wt, include_groups = False)
                     if(isinstance(res, pandas.Series)):
                         res = res.to_frame()
@@ -939,16 +1007,22 @@ def group_avg(df: pandas.DataFrame | polars.DataFrame,
                 res = res.rename(mapping = names)
 
         if(no_merge):
-            if(set_index and is_pandas):
-                res = res.set_index(gr)
-                return(res)
+            if(set_index is not None and is_pandas):
+                    if(isinstance(set_index, str)):
+                        set_index = [set_index]
+                    if(set_index):
+                        fin = res.set_index(set_index)
+                        return(fin)
             return(res)
         else:
             if(is_pandas):
                 res = df.merge(res, how = merge_how, on = gr)
-                if(set_index):
-                    res = res.set_index(gr)
-                    return(res)
+                if(set_index is not None):
+                    if(isinstance(set_index, str)):
+                        set_index = [set_index]
+                    if(set_index):
+                        fin = res.set_index(set_index)
+                        return(fin)
                 return(res)
             else:
                 res = df.join(res, how = merge_how, on = gr)
@@ -961,7 +1035,7 @@ def group_sum(df: pandas.DataFrame | polars.DataFrame,
               name: typing.Optional[typing.Union[str, list[str]]] = None,
               no_merge: typing.Optional[bool] = False,
               merge_how: typing.Optional[str] = 'left',
-              set_index: typing.Optional[bool] = False
+              set_index: typing.Optional[typing.Union[str, list[str]]] = None
             ) -> pandas.DataFrame | polars.DataFrame:
     """
     Groups the DataFrame by specified column(s), calculates the sum of specified column(s)
@@ -978,6 +1052,8 @@ def group_sum(df: pandas.DataFrame | polars.DataFrame,
                                  Default is False.
     - merge_how (str, optional): Method to use when merging the result back to the original DataFrame.
                                  Options are 'left', 'right', 'outer', 'inner'. Default is 'left'.
+    - set_index (str or list[str], optional): If provided, sets the index of the returned DataFrame to these columns.
+                                 
 
     Returns:
     - pandas/polars.DataFrame: The DataFrame with groups summarized by summing the specified columns,
@@ -1058,16 +1134,22 @@ def group_sum(df: pandas.DataFrame | polars.DataFrame,
             res = res.rename(mapping = names)
         
     if(no_merge):
-        if(set_index and is_pandas):
-            res = res.set_index(gr)
-            return(res)
+        if(set_index is not None and is_pandas):
+            if(isinstance(set_index, str)):
+                set_index = [set_index]
+            if(set_index):
+                fin = res.set_index(set_index)
+                return(fin)
         return(res)
     else:
         if(is_pandas):
             res = df.merge(res, how = merge_how, on = gr)
-            if(set_index):
-                res = res.set_index(gr)
-                return(res)
+            if(set_index is not None and is_pandas):
+                if(isinstance(set_index, str)):
+                    set_index = [set_index]
+                if(set_index):
+                    fin = res.set_index(set_index)
+                    return(fin)
             return(res)
         else:
             res = df.join(res, how = merge_how, on = gr)
@@ -1873,8 +1955,11 @@ def sort_portfolios(df: pandas.DataFrame,
     
     _df_internal = df.copy()
     
+    if sorting_funcs.keys() != breakpoints.keys():
+        raise ValueError('keys of sorting_funcs and breakpoints must match.')
+    
     needed_cols: list = [id_col, date_col, return_col]
-    if weight_col is not None:
+    if weight_col is not None and weight_col not in sorting_funcs.keys():
         needed_cols += [weight_col]
     needed_cols += list(set(sorting_funcs.keys()))
     _df_internal = _df_internal[needed_cols]
@@ -1916,8 +2001,129 @@ def sort_portfolios(df: pandas.DataFrame,
     # apply ranking to stocks
     rank_cols = []
     for char, func in sorting_funcs.items():
-        rank_cols.append(f'{char}_rank')
-        rebalance_df[f'{char}_rank'] = rebalance_df.apply(func, args = (char, ), axis = 1)
+        rank_col = f'{char}_rank'
+        rank_cols.append(rank_col)
+
+        # Fast vectorized paths for built-in sorting functions; fallback to apply
+        v = rebalance_df[char]
+
+        # Initialize with '--fail' to mirror existing behavior on unmatched rows
+        out = pandas.Series(numpy.full(len(rebalance_df), '--fail', dtype=object), index=rebalance_df.index)
+
+        if func is sort_50:
+            p50 = rebalance_df[f'{char}_50%']
+            mask1 = v < p50
+            mask2 = v >= p50
+            out[mask1] = f'{char}1'
+            out[mask2] = f'{char}2'
+            rebalance_df[rank_col] = out
+
+        elif func is sort_3070:
+            p30 = rebalance_df[f'{char}_30%']
+            p70 = rebalance_df[f'{char}_70%']
+            mask1 = v < p30
+            mask2 = (v >= p30) & (v < p70)
+            mask3 = v >= p70
+            out[mask1] = f'{char}1'
+            out[mask2] = f'{char}2'
+            out[mask3] = f'{char}3'
+            rebalance_df[rank_col] = out
+
+        elif func is sort_03070:
+            p30 = rebalance_df[f'{char}_30%']
+            p70 = rebalance_df[f'{char}_70%']
+            mask1 = v <= 0
+            mask2 = (v >= 0) & (v < p30)
+            mask3 = (v >= p30) & (v < p70)
+            mask4 = v >= p70
+            out[mask1] = f'{char}1'
+            out[mask2] = f'{char}2'
+            out[mask3] = f'{char}3'
+            out[mask4] = f'{char}4'
+            rebalance_df[rank_col] = out
+
+        elif func is sort_tercile:
+            p33 = rebalance_df[f'{char}_33%']
+            p66 = rebalance_df[f'{char}_66%']
+            mask1 = v <= p33
+            mask2 = (v > p33) & (v <= p66)
+            mask3 = v > p66
+            out[mask1] = f'{char}1'
+            out[mask2] = f'{char}2'
+            out[mask3] = f'{char}3'
+            rebalance_df[rank_col] = out
+
+        elif func is sort_quartile:
+            p25 = rebalance_df[f'{char}_25%']
+            p50 = rebalance_df[f'{char}_50%']
+            p75 = rebalance_df[f'{char}_75%']
+            mask1 = v <= p25
+            mask2 = (v > p25) & (v <= p50)
+            mask3 = (v > p50) & (v <= p75)
+            mask4 = v > p75
+            out[mask1] = f'{char}1'
+            out[mask2] = f'{char}2'
+            out[mask3] = f'{char}3'
+            out[mask4] = f'{char}4'
+            rebalance_df[rank_col] = out
+
+        elif func is sort_quintile:
+            p20 = rebalance_df[f'{char}_20%']
+            p40 = rebalance_df[f'{char}_40%']
+            p60 = rebalance_df[f'{char}_60%']
+            p80 = rebalance_df[f'{char}_80%']
+            mask1 = v <= p20
+            mask2 = (v > p20) & (v <= p40)
+            mask3 = (v > p40) & (v <= p60)
+            mask4 = (v > p60) & (v <= p80)
+            mask5 = v > p80
+            out[mask1] = f'{char}1'
+            out[mask2] = f'{char}2'
+            out[mask3] = f'{char}3'
+            out[mask4] = f'{char}4'
+            out[mask5] = f'{char}5'
+            rebalance_df[rank_col] = out
+
+        elif func is sort_decile:
+            p10 = rebalance_df[f'{char}_10%']
+            p20 = rebalance_df[f'{char}_20%']
+            p30 = rebalance_df[f'{char}_30%']
+            p40 = rebalance_df[f'{char}_40%']
+            p50 = rebalance_df[f'{char}_50%']
+            p60 = rebalance_df[f'{char}_60%']
+            p70 = rebalance_df[f'{char}_70%']
+            p80 = rebalance_df[f'{char}_80%']
+            p90 = rebalance_df[f'{char}_90%']
+            masks = [
+                v < p10,
+                (v >= p10) & (v < p20),
+                (v >= p20) & (v < p30),
+                (v >= p30) & (v < p40),
+                (v >= p40) & (v < p50),
+                (v >= p50) & (v < p60),
+                (v >= p60) & (v < p70),
+                (v >= p70) & (v < p80),
+                (v >= p80) & (v < p90),
+                v >= p90,
+            ]
+            labels = [f'{char}{i}' for i in range(1, 11)]
+            for m, lab in zip(masks, labels):
+                out[m] = lab
+            rebalance_df[rank_col] = out
+
+        elif func is sort_050:
+            p50 = rebalance_df[f'{char}_50%']
+            mask1 = v < 0
+            mask2 = (v >= 0) & (v < p50)
+            mask3 = v >= p50
+            out[mask1] = f'{char}1'
+            out[mask2] = f'{char}2'
+            out[mask3] = f'{char}3'
+            rebalance_df[rank_col] = out
+
+        else:
+            # Fallback for custom callables
+            rebalance_df[rank_col] = rebalance_df.apply(func, args=(char,), axis=1)
 
     # remove stocks that could not be sorted
     for rank_col in rank_cols:
@@ -1957,7 +2163,7 @@ def sort_portfolios(df: pandas.DataFrame,
     firm = group_nunique(fin, 
                          gr = [date_col, 'port_name'], 
                          vr = id_col, 
-                         name = 'num_firms', 
+                         name = {id_col: 'num_firms'}, 
                          no_merge = True
                         )
     
@@ -2008,7 +2214,7 @@ def sort_050(row: pandas.Series, var: str) -> str:
     """
     if(row[var] < 0):
         res = f'{var}1'
-    if(row[var] >= 0 and row[var] < row[f'{var}_50%']):
+    elif(row[var] >= 0 and row[var] < row[f'{var}_50%']):
         res = f'{var}2'
     elif(row[var] >= row[f'{var}_50%']):
         res = f'{var}3'
@@ -2176,4 +2382,3 @@ def sort_decile(row: pandas.Series, var: str) -> str:
 
 
 
-
