@@ -209,7 +209,6 @@ class WebData:
         # set link bounds
         res_df = res_df[(res_df.date >= res_df.linkdt) & (res_df.date <= res_df.linkenddt)]
         
-        
         # divide quarterly earnings by 3 to get monthly earnings
         if freq == 'M':
             res_df['earn'] = numpy.where(res_df.earn.isnull(), numpy.nan, res_df.earn / 3)
@@ -291,7 +290,7 @@ class WebData:
             ids=gvkeys, 
             fields=['gvkey', 'datadate', 'fyearq', 'seqq', 'txditcq', 'pstkrq',
                     'pstkq', 'ibq', 'atq', 'ltq'], 
-            table_name='COMPA.FUNDQ', 
+            table_name='COMP.FUNDQ', 
             date_var='datadate',
             start_date=start_date, 
             end_date=end_date))
